"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getResArray = getResArray;
exports.filterOptions = filterOptions;
const architect_1 = require("@angular-devkit/architect");
const fs = require("fs");
const path = require("path");
const webpack = require("webpack");
const Mustache = require("mustache");
__exportStar(require("./schema"), exports);
/**
 * @param json
 */
function getResArray(assetGroups) {
    const resArray = [];
    assetGroups.filter((assetGroup) => assetGroup.installMode === 'prefetch')
        .forEach((assetGroup) => resArray.push(...assetGroup.urls));
    return resArray;
}
/**
 * @param options prefetch Builder Options
 * @param allowedOptions list of allowed options for Builder configuration
 */
function filterOptions(options, allowedOptions) {
    return Object.keys(options)
        .filter(key => allowedOptions.includes(key))
        .reduce((obj, key) => {
        return Object.assign(Object.assign({}, obj), { [key]: options[key] });
    }, {});
}
/** Maximum number of steps */
const STEP_NUMBER = 4;
/**
 * Compute the path where to find the swJson file from the ngx-prefetch and the build target options.
 * @param targetBuildBuilder
 * @param targetOutputPath
 * @param outputPathOverride
 */
function computeSwJsonFolderPath(targetBuildBuilder, targetOutputPath, outputPathOverride) {
    if (typeof outputPathOverride === 'string' && !!outputPathOverride) {
        return outputPathOverride;
    }
    if (typeof targetOutputPath === 'string') {
        if (['@nx/angular:application', '@angular-devkit/build-angular:application'].indexOf(targetBuildBuilder) > -1) {
            return path.join(targetOutputPath, 'browser');
        }
        return targetOutputPath;
    }
    if (typeof targetOutputPath.browser === 'string' && typeof targetOutputPath.base === 'string') {
        return path.join(targetOutputPath.base, targetOutputPath.browser);
    }
    if (typeof targetOutputPath.base === 'string') {
        return targetOutputPath.base;
    }
    return undefined;
}
exports.default = (0, architect_1.createBuilder)((options, context) => __awaiter(void 0, void 0, void 0, function* () {
    context.reportRunning();
    context.reportProgress(0, STEP_NUMBER, 'Retrieve production output folder.');
    const [project, target, configuration] = options.targetBuild.split(':');
    const targetBuild = { project, target, configuration };
    const [targetBuildRawOptions, targetBuildBuilder] = yield Promise.all([
        context.getTargetOptions(targetBuild),
        context.getBuilderNameForTarget(targetBuild)
    ]);
    const targetBuildOptions = yield context.validateOptions(targetBuildRawOptions, targetBuildBuilder);
    /** Path to the build output folder */
    const prodBuildOutputPath = computeSwJsonFolderPath(targetBuildBuilder, targetBuildOptions.outputPath, options.outputPath);
    /** Check the minimum of mandatory options to the builders */
    if (!prodBuildOutputPath) {
        return {
            success: false,
            error: `Neither the ngx-prefetch task nor targetBuild ${options.targetBuild} provide an 'outputPath' option.`
        };
    }
    context.reportProgress(1, STEP_NUMBER, 'Read ngsw.json');
    let swJsonString;
    try {
        swJsonString = fs.readFileSync(path.join(process.cwd(), prodBuildOutputPath, 'ngsw.json'), { encoding: 'utf-8' });
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
    }
    catch (error) {
        return {
            success: false,
            error: error.message
        };
    }
    const swJson = JSON.parse(swJsonString);
    if (!swJson.assetGroups || swJson.assetGroups.length <= 0) {
        return {
            success: false,
            error: 'Prefetch builder failed. No asset group found in the Angular Service Worker.'
        };
    }
    const resourceArray = getResArray(swJson.assetGroups);
    context.reportProgress(2, STEP_NUMBER, 'Read prefetch template file.');
    const prefetchTemplate = fs.readFileSync(path.join(__dirname, 'templates', 'prefetch.mustache'), { encoding: 'utf-8' });
    const configOptions = ['crossorigin', 'resourceTypes', 'fallbackLocalesMap'];
    const variables = {
        resourceArray: JSON.stringify(resourceArray),
        prefetchConfig: JSON.stringify(filterOptions(options, configOptions)),
        staticsFullPath: options.staticsFullPath,
        localizationPattern: options.localizationPattern
    };
    const prefetchJs = Mustache.render(prefetchTemplate, variables);
    context.reportProgress(3, STEP_NUMBER, 'Write prefetch js script.');
    const tempOutputPath = path.join(process.cwd(), prodBuildOutputPath, 'tempPrefetch.js');
    fs.writeFileSync(tempOutputPath, prefetchJs);
    context.reportProgress(4, STEP_NUMBER, 'Webpack');
    const compiler = webpack({
        mode: options.production ? 'production' : 'none',
        entry: tempOutputPath,
        output: {
            path: path.join(process.cwd(), prodBuildOutputPath),
            filename: 'ngxPrefetch.js'
        },
        devtool: options.production ? 'source-map' : false
    });
    return new Promise((resolve) => {
        compiler.run((err, stats) => {
            fs.unlinkSync(tempOutputPath);
            if (err || stats.hasErrors()) {
                resolve({
                    success: false,
                    error: `Webpack transpilation failed. ${err || stats.hasErrors()}`
                });
            }
            else {
                resolve({
                    success: true
                });
            }
        });
    });
}));
//# sourceMappingURL=index.js.map