"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ngAdd = ngAdd;
const schematics_1 = require("@angular-devkit/schematics");
function getFinalProjectName(workspace, projectName) {
    if (projectName)
        return projectName;
    const projectKeys = Object.keys(workspace.projects);
    if (projectKeys.length !== 1)
        throw new schematics_1.SchematicsException('No project name provided and several projects were found in angular.json.');
    return projectKeys[0];
}
/**
 * Update the application with the prefetch script
 *
 * @param options
 */
function updatePrefetchBuilder(options) {
    return (tree, context) => {
        let workspace;
        try {
            workspace = tree.readJson('/angular.json');
        }
        catch (e) {
            throw new schematics_1.SchematicsException(`Could not parse /angular.json. ${e}`);
        }
        const projectName = getFinalProjectName(workspace, options.projectName);
        const workspaceProject = workspace.projects[projectName];
        const packageJsonPath = `${workspaceProject.root}/package.json`;
        if (!tree.exists(packageJsonPath)) {
            throw new schematics_1.SchematicsException('Could not find package.json');
        }
        const packageJson = tree.readJson(packageJsonPath);
        packageJson.scripts || (packageJson.scripts = {});
        packageJson.scripts['generate:prefetch'] = `ng run ${projectName}:generate-prefetch`;
        tree.overwrite(`${workspaceProject.root}/package.json`, JSON.stringify(packageJson, null, 2));
        workspaceProject.architect || (workspaceProject.architect = {});
        workspaceProject.architect['generate-prefetch'] = {
            builder: '@o3r/ngx-prefetch:run',
            options: {
                targetBuild: `${projectName}:${options.targetBuild}`
            }
        };
        if (!tree.exists('/ngsw-config.json')) {
            context.logger.warn('Run `ng add @angular/pwa` to setup Angular Service Worker. This is a mandatory package for the ngx-prefetch library.');
        }
        workspace.projects[projectName] = workspaceProject;
        tree.overwrite('/angular.json', JSON.stringify(workspace, null, 2));
        return tree;
    };
}
/**
 * Add the ngx-prefetch library to an Angular Project
 *
 * @param options
 */
function ngAdd(options) {
    return (0, schematics_1.chain)([
        updatePrefetchBuilder(options)
    ]);
}
//# sourceMappingURL=index.js.map